/**********************************************************************/
/**
 * @file MB9BF506_serial.c
 * ITF-SingleHost TvVAW[
 *
 * @author T.Tanaka
 * $Date: 2013/04/01 $
 * $Revision: $
 *
 * Copyright (c) 2013 Interface Inc. All Rights reserved.
 *
 * <pre>
 * --------------------------------------------------------------------
 * ύX
 *  +-------------- ԍ (000 ` 999)
 *  |	  +--------- CĂVXeo[W
 *  |	  | 	+--- VKAύXAǉA폜̕
 *  v	  v 	v
 *  No  Ver   N	  O			
 * ---+-----+----+----------+-------------+----------------------------
 * 000 01.00 VK 2013/04/01 ITF Tanaka    VK쐬
 * ---+-----+----+----------+-------------+----------------------------
 * </pre>
 */
/**********************************************************************/

/*------------------------------------------*/
/*				CN[h				*/
/*------------------------------------------*/
#include "itflib.h"
#include "serial.h"

/*------------------------------------------*/
/*				錾						*/
/*------------------------------------------*/
/**
 * @brief	VA|[g
 */
#define MAX_SERIAL_NUM	(sizeof(g_reg_table) / sizeof(g_reg_table[0]))

/**
 * @brief	ytFNbN [Hz]
 */
#define PCLK			40000000

/**
 * @name	|[gWX^`
 * @{
 */
#define PFR(n)			(*(volatile uint32_t*)(0x40033000 | ((n) << 2)))
#define EPFR(n)			(*(volatile uint32_t*)(0x40033600 | ((n) << 2)))
#define ADE				(*(volatile uint32_t*)(0x40033500))
/** @} */

/**
 * @name	VAWX^`
 * @{
 */
#define SCR(n)			((volatile uint8_t *)(0x40038001 | ((n) << 8)))
#define  BIT_UPCL			(0x80u)
#define  BIT_RIE			(0x10u)
#define  BIT_TIE			(0x08u)
#define  BIT_TBIE			(0x04u)
#define  BIT_RXE			(0x02u)
#define  BIT_TXE			(0x01u)

#define SMR(n)			((volatile uint8_t *)(0x40038000 | ((n) << 8)))
#define  BIT_MD				(0xE0u)
#define   BIT_MD_SET(b)			(((unsigned)(b) & 0x07u) << 5)
#define   BIT_MD_GET(b)			(((unsigned)(b) & 0xE0u) >> 5)
#define  BIT_WUCR			(0x10u)
#define  BIT_SBL			(0x08u)
#define  BIT_BDS			(0x04u)
#define  BIT_SOE			(0x01u)

#define SSR(n)			((volatile uint8_t *)(0x40038005 | ((n) << 8)))
#define  BIT_REC			(0x80u)
#define  BIT_PE				(0x20u)
#define  BIT_FRE			(0x10u)
#define  BIT_ORE			(0x08u)
#define  BIT_RDRF			(0x04u)
#define  BIT_TDRE			(0x02u)
#define  BIT_TBI			(0x01u)

#define ESCR(n)			((volatile uint8_t *)(0x40038004 | ((n) << 8)))
#define  BIT_FLWEN			(0x80u)
#define  BIT_ESBL			(0x40u)
#define  BIT_INV			(0x20u)
#define  BIT_PEN			(0x10u)
#define  BIT_P				(0x08u)
#define  BIT_L				(0x07u)
#define   BIT_L_SET(b)			(((unsigned)(b) & 0x07u) << 0)
#define   BIT_L_GET(b)			(((unsigned)(b) & 0x07u) >> 0)

#define DR(n)			((volatile uint16_t*)(0x40038008 | ((n) << 8)))
#define BGR(n)			((volatile uint16_t*)(0x4003800C | ((n) << 8)))

#define FCR(n)			((volatile uint16_t*)(0x40038014 | ((n) << 8)))
#define  BIT_FLSTE			(0x1000u)
#define  BIT_FRIIE			(0x0800u)
#define  BIT_FDRQ			(0x0400u)
#define  BIT_FTIE			(0x0200u)
#define  BIT_FSEL			(0x0100u)
#define  BIT_FLST			(0x0040u)
#define  BIT_FLD			(0x0020u)
#define  BIT_FSET			(0x0010u)
#define  BIT_FCL2			(0x0008u)
#define  BIT_FCL1			(0x0004u)
#define  BIT_FE2			(0x0002u)
#define  BIT_FE1			(0x0001u)

#define FBYTE1(n)		((volatile uint8_t *)(0x40038018 | ((n) << 8)))
#define FBYTE2(n)		((volatile uint8_t *)(0x40038019 | ((n) << 8)))
/** @} */

/**
 * @brief	WX^e[u
 */
typedef struct
{
	volatile uint8_t	*SMR;
	volatile uint8_t	*SCR;
	volatile uint8_t	*ESCR;
	volatile uint8_t	*SSR;
	volatile uint16_t	*DR;
	volatile uint16_t	*BGR;
	volatile uint16_t	*FCR;
	volatile uint8_t	*FBYTE2;
} UART_REG;

/*------------------------------------------*/
/*				ϐ					*/
/*------------------------------------------*/
/**
 * @brief	WX^e[u
 */
static const UART_REG g_reg_table[] =
{
	{SMR(0), SCR(0), ESCR(0), SSR(0), DR(0), BGR(0), NULL,   NULL     },
	{SMR(1), SCR(1), ESCR(1), SSR(1), DR(1), BGR(1), NULL,   NULL     },
	{SMR(2), SCR(2), ESCR(2), SSR(2), DR(2), BGR(2), NULL,   NULL     },
	{SMR(3), SCR(3), ESCR(3), SSR(3), DR(3), BGR(3), NULL,   NULL     },
	{SMR(4), SCR(4), ESCR(4), SSR(4), DR(4), BGR(4), FCR(4), FBYTE2(4)},
	{SMR(5), SCR(5), ESCR(5), SSR(5), DR(5), BGR(5), FCR(5), FBYTE2(5)},
	{SMR(6), SCR(6), ESCR(6), SSR(6), DR(6), BGR(6), FCR(6), FBYTE2(6)},
	{SMR(7), SCR(7), ESCR(7), SSR(7), DR(7), BGR(7), FCR(7), FBYTE2(7)},
};

/**
 * @brief	f[^IrbgɃZbgl
 */
static const uint8_t g_length_table[] =
{
	BIT_L_SET(1),	/* 5bit */
	BIT_L_SET(2),	/* 6bit */
	BIT_L_SET(3),	/* 7bit */
	BIT_L_SET(0),	/* 8bit */
	BIT_L_SET(4)	/* 9bit */
};

/**
 * @brief	M҂ɌĂԊ֐
 */
static SERIAL_POLLING g_polling[MAX_SERIAL_NUM];

/**********************************************************************/
/**
 * @brief	VA
 *
 * @param	id			[in]	VAID
 * @param	baudrate	[in]	{[[g
 * @param	bit			[in]	f[^rbg (5`9)
 * @param	parity		[in]	peB (0:Ȃ 1: 2:)
 * @param	stop		[in]	Xgbvrbg (1`4)
 * @param	polling		[in]	M҂ɌĂԊ֐
 *
 * @retval	TRUE		I
 * @retval	FALSE		ُI
 */
ibool Serial_Init(uint8_t id, uint32_t baudrate, uint8_t bit, uint8_t parity, uint8_t stop, SERIAL_POLLING polling)
{
	uint16_t		bgr_val = (PCLK - (baudrate / 2)) / baudrate;
	uint8_t			smr_val;
	uint8_t			escr_val;
	const UART_REG	*reg;

	if ((id < MAX_SERIAL_NUM) && (bgr_val < 0x8000) && (5 <= bit) && (bit <= 9) && (1 <= stop) && (stop <= 4) && (parity <= 2))
	{
		reg = &g_reg_table[id];

		/* VA֎~ */
		*reg->SCR = 0;

		if (*reg->FCR != NULL)
		{
			/* FIFO֎~ */
			*reg->FCR &= ~(BIT_FE2 | BIT_FE1);
		}

		/* ϐ */
		g_polling[id] = polling;

		/* |[g@\ݒ */
		PFR(0) |= 0x00000C00;	/* SIN4_0, SOT4_0	*/
		EPFR(8) = 0x00000050;	/* SIN4_0, SOT4_0	*/

		/* vO}uNA */
		*reg->SCR |= BIT_UPCL;

		/* SMREESCREBGRݒ */
		smr_val  = BIT_MD_SET(0) | BIT_SOE;
		escr_val = g_length_table[bit - 5];
		if (stop == 2)
		{
			smr_val |= BIT_SBL;
		}
		else if (stop == 3)
		{
			escr_val |= BIT_ESBL;
		}
		else if (stop == 4)
		{
			smr_val |= BIT_SBL;
			escr_val |= BIT_ESBL;
		}
		if (parity == 1)
		{
			escr_val |= (BIT_PEN | BIT_P);
		}
		else if (parity == 2)
		{
			escr_val |= BIT_PEN;
		}
		*reg->SMR  = smr_val;
		*reg->ESCR = escr_val;
		*reg->BGR  = bgr_val;

		if (*reg->FCR != NULL)
		{
			/* MFIFO */
			*reg->FCR |= (BIT_FCL2 | BIT_FE2);
			*reg->FBYTE2 = 1;
		}

		/* VAJn */
		*reg->SCR = BIT_RXE | BIT_TXE;

		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/**********************************************************************/
/**
 * @brief	񑗐M
 *
 * @param	id	[in]	VAID
 * @param	buf	[in]	Mobt@ւ̃|C^
 *
 * @retval	TRUE		I
 * @retval	FALSE		ُI
 */
ibool Serial_Puts(uint8_t id, const char* buf)
{
	const UART_REG	*reg;

	if ((id < MAX_SERIAL_NUM) && buf && *buf)
	{
		reg = &g_reg_table[id];
		while (*buf)
		{
			*reg->DR = *buf++;
			while (!(*reg->SSR & BIT_TDRE))
			{
				if (g_polling[id])
				{
					g_polling[id]();
				}
			}
		}
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/**********************************************************************/
/**
 * @brief	1M
 *
 * @param	id	[in]	VAID
 * @param	buf	[out]	Mobt@ւ̃|C^
 *
 * @retval	TRUE		I
 * @retval	FALSE		ُI
 */
ibool Serial_Get(uint8_t id, char* buf)
{
	const UART_REG	*reg;

	if ((id < MAX_SERIAL_NUM) && buf)
	{
		reg = &g_reg_table[id];
		if (*reg->SSR & BIT_RDRF)
		{
			*buf = *reg->DR;
			return TRUE;
		}
		if (*reg->SSR & (BIT_PE | BIT_ORE | BIT_FRE))
		{
			*reg->SSR = BIT_REC;
		}
	}

	return FALSE;
}
