/**********************************************************************/
/**
 * @file MB9BF506_timer.c
 * ITF-SingleHost Tv^C}W[
 *
 * @author T.Tanaka
 * $Date: 2013/04/02 $
 * $Revision: $
 *
 * Copyright (c) 2013 Interface Inc. All Rights reserved.
 *
 * <pre>
 * --------------------------------------------------------------------
 * ύX
 *  +-------------- ԍ (000 ` 999)
 *  |	  +--------- CĂVXeo[W
 *  |	  | 	+--- VKAύXAǉA폜̕
 *  v	  v 	v
 *  No  Ver   N	  O			
 * ---+-----+----+----------+-------------+----------------------------
 * 000 01.00 VK 2013/04/02 ITF Tanaka    VK쐬
 * ---+-----+----+----------+-------------+----------------------------
 * </pre>
 */
/**********************************************************************/

/*------------------------------------------*/
/*				CN[h				*/
/*------------------------------------------*/
#include "itflib.h"
#include "timer.h"

/*------------------------------------------*/
/*				錾						*/
/*------------------------------------------*/
/**
 * @brief	TimerPre ݒ (0 ` 2)
 */
#define TIMER_PRE			0

/**
 * @brief	1ms ɑΉJEg
 *
 *			ytFNbN TimerPre ̐ݒ肩vZ.
 *			JEgNbN ytFNbN / (2^(TimerPre * 4)) [MHz] Ȃ̂,
 *			ytFNbN = 40MHz, TimerPre = 0 ̂ƂJEgNbN
 *			40000kHz ƂȂ, 40000JEg 1ms ƂȂ.
 */
#define TIMER_COUNT_1ms		40000

/**
 * @name	^C}WX^`
 * @{
 */
#define TIMER_LOAD			(*(volatile uint32_t*)0x40015000)
#define TIMER_CONTROL		(*(volatile uint32_t*)0x40015008)
#define   BIT_TIMER_SIZE		(0x02u)
#define   BIT_TIMER_PRE			(TIMER_PRE << 2)
#define   BIT_INT_ENABLE		(0x20u)
#define   BIT_TIMER_MODE		(0x40u)
#define   BIT_TIMER_EN			(0x80u)
#define TIMER_INTCLR		(*(volatile uint32_t*)0x4001500C)
#define SETENA				(*(volatile uint32_t*)0xE000E100)
#define   BIT_SETENA			(0x40u)
/** @} */

/*------------------------------------------*/
/*				ϐ					*/
/*------------------------------------------*/
/**
 * @brief	^C}R[obN֐|C^
 */
static TIMER_CALLBACK	g_callback;

/**
 * @brief	JEg
 */
static uint32_t g_count;

/**********************************************************************/
/**
 * @brief	^C}
 *
 * @param	period	 [ms]
 * @param	pf		R[obN
 *
 * @retval	TRUE	
 * @retval	FALSE	s
 */
ibool Timer_Init(uint16_t period, TIMER_CALLBACK callback)
{
	if (callback && !(TIMER_CONTROL & BIT_TIMER_EN))
	{
		g_callback = callback;
		g_count = 0;
		TIMER_CONTROL = BIT_TIMER_SIZE | BIT_TIMER_PRE | BIT_INT_ENABLE | BIT_TIMER_MODE;
		TIMER_LOAD = (period * TIMER_COUNT_1ms) - 1;
		SETENA |= BIT_SETENA;
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/**********************************************************************/
/**
 * @brief	^C}Jn
 *
 * @retval	TRUE	
 * @retval	FALSE	s
 */
ibool Timer_Start(void)
{
	if (!(TIMER_CONTROL & BIT_TIMER_EN))
	{
		TIMER_CONTROL |= BIT_TIMER_EN;
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/**********************************************************************/
/**
 * @brief	^C}I
 *
 * @retval	TRUE	
 * @retval	FALSE	s
 */
ibool Timer_Stop(void)
{
	if (TIMER_CONTROL & BIT_TIMER_EN)
	{
		TIMER_CONTROL &= ~BIT_TIMER_EN;
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/**********************************************************************/
/**
 * @brief	eBbNJEg擾
 *
 * @retval	TRUE	
 * @retval	FALSE	s
 */
uint32_t Timer_GetTickCount(void)
{
	return g_count;
}

/**********************************************************************/
/**
 * @brief	^C}
 */
void Timer_Interrupt(void)
{
	TIMER_INTCLR = 1;
	g_count++;
	if (g_callback)
	{
		g_callback();
	}
}
