/**********************************************************************/
/**
 * @file Sample.c
 * ITF-SingleHost TvAvP[V
 *
 * @author T.Tanaka
 * $Date: 2013/03/27 $
 * $Revision: $
 *
 * Copyright (c) 2013 Interface Inc. All Rights reserved.
 *
 * <pre>
 * --------------------------------------------------------------------
 * ύX
 *  +-------------- ԍ (000 ` 999)
 *  |	  +--------- CĂVXeo[W
 *  |	  | 	+--- VKAύXAǉA폜̕
 *  v	  v 	v
 *  No  Ver   N	  O			
 * ---+-----+----+----------+-------------+----------------------------
 * 000 01.00 VK 2013/03/27 ITF Tanaka    VK쐬
 * ---+-----+----+----------+-------------+----------------------------
 * </pre>
 */
/**********************************************************************/

/*------------------------------------------*/
/*				CN[h				*/
/*------------------------------------------*/
#include <Fujitsu/iomb9bf506.h>
#include "itflib.h"
#include "singlehost.h"
#include "itffile.h"
#include "serial.h"
#include "timer.h"
#include "lcd.h"

/*------------------------------------------*/
/*				錾						*/
/*------------------------------------------*/
/**
 * @name VAݒ
 */
#define SERIAL_CH		4				/**< `lԍ			*/
#define BAUDRATE		19200			/**< {[[g 19200bps	*/
#define DATABIT			8				/**< f[^rbg 8bit	*/
#define PARITY			0				/**< peBȂ			*/
#define STOPBIT			1				/**< Xgbvrbg 1bit	*/
/** @} */

#define DEVICE_NUMBER	1				/**< foCXʔԍ		*/
#define WORK_BUFF_SIZE	(32 * 1024)		/**< [Nobt@TCY	*/
#define FILE_SIZE		(1024 * 1024)	/**< t@CTCY			*/
#define FILE_PATH		"test.bin"		/**< t@C				*/

#define PSW_DETECT_TIME	50				/**< XCb`̉莞	*/
#define PSW()			(PDIR5 & 3 ^ 3)	/**< XCb`̏			*/

#define ABORT()			while (1);		/**< ُI}N			*/

/*------------------------------------------*/
/*				ϐ					*/
/*------------------------------------------*/
static char		g_WorkBuff[WORK_BUFF_SIZE];
static ibool	g_suspend;
static uint32_t	g_pswValue;
static uint32_t	g_pswState;
static uint32_t	g_pswTime;

/*------------------------------------------*/
/*				O֐					*/
/*------------------------------------------*/
extern void wait_us(uint32_t us);

/*------------------------------------------*/
/*				֐					*/
/*------------------------------------------*/
static void File_Callback(int8_t deviceno, int8_t letter, int8_t inout);
static void polling(void);
static void suspend(void);
static void wakeup(void);
static void read_file(void);
static void write_file(void);
static void WDT_Disable(void);
static void Clock_Init(void);

/**********************************************************************/
/**
 * @brief	C֐
 */
void main(void)
{
	ISTATUS		status;
	uint32_t	psw;
	char		c;

	/* n[hEFAEHb`hbO^C}~ */
	WDT_Disable();

	/* NbN */
	Clock_Init();

	/* LCD */
	Lcd_Init();

	/* UART */
	if (!Serial_Init(SERIAL_CH, BAUDRATE, DATABIT, PARITY, STOPBIT, polling))
	{
		ABORT();
	}

	/* ^C} */
	if (!Timer_Init(NUSB_TIMER_INTERVAL, Usb_Host_Notify_Timer))
	{
		ABORT();
	}
	if (!Timer_Start())
	{
		ABORT();
	}

	/* USB Host */
	status = Usb_Host_Init();
	if (status != 0)
	{
		ABORT();
	}

	/* ^Cg\ (CONNECT/DISCONNECT̕\Ɣ̂邽߁Adisk_mountO) */
	Serial_Puts(SERIAL_CH, "\r\nITF-EasyHost I Sample Program\r\n");
	Lcd_Puts(1, 0, "ITF-EasyHost I");
	Lcd_Puts(1, 1, "Sample Program");

	/* t@CVXe */
	status = ITF_FileSystemInit();
	if (status != 0)
	{
		ABORT();
	}
	status = disk_mount(Scd_Driver, DEVICE_NUMBER, __MIN_LETTER__, File_Callback, DISK_REMOVABLE);
	if (status != 0)
	{
		ABORT();
	}

	/* C[v */
	while (TRUE)
	{
		if (!Serial_Get(SERIAL_CH, &c))
		{
			c = 0;
			if (psw != g_pswState)
			{
				psw = g_pswState;
				if (psw == 1)
				{
					c = 'w';
				}
				else if (psw == 2)
				{
					c = 'r';
				}
			}
		}

		switch (c)
		{
		case 'S':
			suspend();
			break;

		case 'W':
			wakeup();
			break;

		case 'r':
			read_file();
			break;

		case 'w':
			write_file();
			break;
		}

		/* |[O */
		polling();
	}
}

/**********************************************************************/
/**
 * @brief	ITF-EasyHost R[obN֐
 *
 * @param	dev			[in]	foCXnh
 * @param	eventnum	[in]	Cxgԍ
 * @param	param		[in]	Cxgp[^
 */
void Event_Callback(ihandle dev, int32_t eventnum, void* param)
{
	if (dev == 0)
	{
		switch (eventnum)
		{
		case NC_EVENT_CONNECT:
		case NC_EVENT_DISCONNECT:
		case NC_EVENT_WAKEUP_REQUEST:
			g_suspend = FALSE;
			break;

		case NC_EVENT_SUSPEND_COMPLETE:
			g_suspend = TRUE;
			break;
		}
	}
}

/**********************************************************************/
/**
 * @brief	ITF-FILE R[obN֐
 *
 * @param	deviceno	[in]	foCXʔԍ
 * @param	letter		[in]	ʒmꂽhCu^[
 * @param	inout		[in]	}ʒm(}:EVENT_CONNECTA:EVENT_DISCONNECT)
 */
static void File_Callback(int8_t deviceno, int8_t letter, int8_t inout)
{
	if (inout == EVENT_CONNECT)
	{
		Serial_Puts(SERIAL_CH, "CONNECT\r\n");
		Lcd_Clear();
		Lcd_Puts(0, 0, "CONNECT");
	}
	else if (inout == EVENT_DISCONNECT)
	{
		Serial_Puts(SERIAL_CH, "DISCONNECT\r\n");
		Lcd_Clear();
		Lcd_Puts(0, 0, "DISCONNECT");
	}
}

/**********************************************************************/
/**
 * @brief	|[O
 */
static void polling(void)
{
	uint32_t	psw = PSW();

	/* XCb`̃`^O */
	psw = PSW();
	if (g_pswValue != psw)
	{
		g_pswValue = psw;
		g_pswTime = Timer_GetTickCount();
	}
	else if (Timer_GetTickCount() - g_pswTime >= PSW_DETECT_TIME / NUSB_TIMER_INTERVAL)
	{
		g_pswState = psw;
	}

	/* Cxg|[O */
	Usb_Host_Poll_Event();
}

/**********************************************************************/
/**
 * @brief	TXyh
 */
static void suspend(void)
{
	ISTATUS	status;

	if (!g_suspend)
	{
		status = Usb_Host_Suspend();
		if (status == 0)
		{
			g_suspend = TRUE;
		}
		else if (status == NE_PENDING)
		{
			while (!g_suspend)
			{
				Usb_Host_Poll_Event();
			}
		}
		if (g_suspend)
		{
			Serial_Puts(SERIAL_CH, "SUSPEND\r\n");
			wait_us(500000);
			Lcd_Clear();
			Lcd_Puts(0, 0, "SUSPEND");
		}
	}
}

/**********************************************************************/
/**
 * @brief	EFCNAbv
 */
static void wakeup(void)
{
	ISTATUS	status;

	if (g_suspend)
	{
		status = Usb_Host_Wakeup();
		if (status == 0)
		{
			g_suspend = FALSE;
		}
		else if (status == NE_PENDING)
		{
			while (g_suspend)
			{
				Usb_Host_Poll_Event();
			}
		}
		if (!g_suspend)
		{
			Serial_Puts(SERIAL_CH, "WAKEUP\r\n");
			Lcd_Clear();
			Lcd_Puts(0, 0, "WAKEUP");
			wait_us(500000);
		}
	}
}

/**********************************************************************/
/**
 * @brief	t@Cǂݍ
 */
static void read_file(void)
{
	uint32_t	i;
	uint32_t	size;
	uint32_t	time;
	uint32_t	speed;
	EXTFCB		*pfcb;
	ISTATUS		status;
	ISTATUS		status2;

	Lcd_Clear();
	Lcd_Puts(0, 0, "reading...");

	/* ^C}Jn */
	Timer2_Start();

	/* I[v */
	status = f_open(&pfcb, FILE_PATH, O_RDWR, S_IREAD | S_IWRITE);
	if (status == 0)
	{
		/* t@Cǂݍ */
		for (i = 0; status == 0; i += size)
		{
			status = f_read(pfcb, (uint8_t*)g_WorkBuff, sizeof(g_WorkBuff), &size);
		}

		/* N[Y */
		status2 = f_close(pfcb);
	}

	/* ^C}~ */
	time = Timer2_Stop();

	/* ʂ\ */
	Lcd_Clear();
	if ((status == FILES_END_OF_FILE) && (status2 == 0))
	{
		speed = (uint32_t)(i * 1000000ULL / time);
		sprintf(g_WorkBuff, "READ:  %lu[byte] %lu[us] %lu[byte/s]\r\n", i, time, speed);
		Serial_Puts(SERIAL_CH, g_WorkBuff);

		sprintf(g_WorkBuff, "%9lu us", time);
		Lcd_Puts(0, 0, g_WorkBuff);
		sprintf(g_WorkBuff, "%9lu byte/s", speed);
		Lcd_Puts(0, 1, g_WorkBuff);
	}
	else
	{
		Serial_Puts(SERIAL_CH, "READ:  failed\r\n");
		Lcd_Puts(0, 0, "READ failed.");
	}
}

/**********************************************************************/
/**
 * @brief	t@C
 */
static void write_file(void)
{
	uint32_t	i;
	uint32_t	size;
	uint32_t	time;
	uint32_t	speed;
	EXTFCB		*pfcb;
	ISTATUS		status;
	ISTATUS		status2;

	Lcd_Clear();
	Lcd_Puts(0, 0, "writing...");

	/* ݃f[^ */
	for (i = 0; i < sizeof(g_WorkBuff); i++)
	{
		g_WorkBuff[i] = (char)(i & 0xFF);
	}

	/* ^C}Jn */
	Timer2_Start();

	/* I[v */
	status = f_open(&pfcb, FILE_PATH, O_CREAT | O_RDWR, S_IREAD | S_IWRITE);
	if (status == 0)
	{
		/* t@C */
		for (i = 0; i < FILE_SIZE; i += size)
		{
			status = f_write(pfcb, (uint8_t*)g_WorkBuff, min(FILE_SIZE - i, sizeof(g_WorkBuff)), &size);
			if (status != 0)
			{
				break;
			}
		}

		/* N[Y */
		status2 = f_close(pfcb);
	}

	/* ^C}~ */
	time = Timer2_Stop();

	/* ʂ\ */
	Lcd_Clear();
	if ((status == 0) && (status2 == 0))
	{
		speed = (uint32_t)(i * 1000000ULL / time);
		sprintf(g_WorkBuff, "WRITE: %lu[byte] %lu[us] %lu[byte/s]\r\n", i, time, speed);
		Serial_Puts(SERIAL_CH, g_WorkBuff);

		sprintf(g_WorkBuff, "%9lu us", time);
		Lcd_Puts(0, 0, g_WorkBuff);
		sprintf(g_WorkBuff, "%9lu byte/s", speed);
		Lcd_Puts(0, 1, g_WorkBuff);
	}
	else
	{
		Serial_Puts(SERIAL_CH, "WRITE: failed\r\n");
		Lcd_Puts(0, 0, "WRITE failed.");
	}
}

/**********************************************************************/
/**
 * @brief	n[hEFAEHb`hbO^C}~
 */
static void WDT_Disable(void)
{
	WDG_LCK = 0x1ACCE551; 
	WDG_LCK = 0xE5331AAE;
	WDG_CTL = 0x00;
}

/**********************************************************************/
/**
 * @brief	NbN
 */
static void Clock_Init(void)
{
	/* oXvXP[ݒ */
	BSC_PSR   = 0x00;			/* x[XNbN   1/1	*/
	APBC0_PSR = 0x01;			/* APB0oXNbN 1/2	*/
	APBC1_PSR = 0x81;			/* APB1oXNbN 1/2	*/
	APBC2_PSR = 0x81;			/* APB2oXNbN 1/2	*/

	/* CNbNU҂Ԑݒ (2ms) */
	CSW_TMR = 0x09;

	/* CNbNU */
	SCM_CTL_bit.MOSCE = 1;

	/* CNbNU҂ */
	while(!SCM_STR_bit.MORDY);

	/* PLLNbNU҂Ԑݒ (512us) */
	PSW_TMR = 0x02;

	/* PLL{ݒ */
	PLL_CTL1 = 0x00;			/* K=1, M=1				*/
	PLL_CTL2 = 0x13;			/* N=20					*/

	/* PLLU */
	SCM_CTL_bit.PLLE = 1;

	/* PLLU҂ */
	while(!SCM_STR_bit.PLRDY);

	/* CPLLNbNI */
	SCM_CTL_bit.RCS = 2;

	/* NbN؂ւ҂ */
	while(SCM_STR_bit.RCM != 2);
}
