/**********************************************************************/
/**
 * @file lcd.c
 * ITF-SingleHost TvLCDW[.
 *
 * @author T.Tanaka
 * $Date: 2013/04/01 $
 * $Revision: $
 *
 * Copyright (c) 2013 Interface Inc. All Rights reserved.
 *
 * <pre>
 * --------------------------------------------------------------------
 * ύX
 *  +-------------- ԍ (000 ` 999)
 *  |	  +--------- CĂVXeo[W
 *  |	  | 	+--- VKAύXAǉA폜̕
 *  v	  v 	v
 *  No  Ver   N	  O			
 * ---+-----+----+----------+-------------+----------------------------
 * 000 01.00 VK 2013/04/01 ITF Tanaka    VK쐬
 * ---+-----+----+----------+-------------+----------------------------
 * </pre>
 */
/**********************************************************************/

/*------------------------------------------*/
/*				CN[h				*/
/*------------------------------------------*/
#include <Fujitsu\iomb9bf506.h>
#include "itflib.h"
#include "lcd.h"

/*------------------------------------------*/
/*				錾						*/
/*------------------------------------------*/
#define LCD_COLUMN_MAX			16		/**< 							*/
#define LCD_ROW_MAX				2		/**< s							*/
#define LCD_BUSY_TIMEOUT		2000	/** rW[҂̃^CAEg	*/

/**
 * @name	|[g`
 * @{
 */
#define LCD_INIT()				\
	do							\
	{							\
		PDOR2 &= ~0x0100;		\
		DDR2  |=  0x0100;		\
		PFR2  &= ~0x01FF;		\
		DDR4  |=  0x0300;		\
		PFR4  &= ~0x0300;		\
	} while (0)

#define LCD_DIR_IN()			(*(volatile unsigned char *)&DDR2 = 0x00)
#define LCD_DIR_OUT()			(*(volatile unsigned char *)&DDR2 = 0xFF)
#define LCD_READ()				(*(volatile unsigned char *)&PDIR2)
#define LCD_WRITE(a)			(*(volatile unsigned char *)&PDOR2 = (a))
#define LCD_E_HIGH()			PDOR2 |=  (1 << 8)
#define LCD_E_LOW()				PDOR2 &= ~(1 << 8)
#define LCD_RS_HIGH()			PDOR4 |=  (1 << 8)
#define LCD_RS_LOW()			PDOR4 &= ~(1 << 8)
#define LCD_RW_HIGH()			PDOR4 |=  (1 << 9)
#define LCD_RW_LOW()			PDOR4 &= ~(1 << 9)
/** @} */

/**
 * @name	EFCg
 * @{
 */
#define LCD_POWER_ON_WAIT		30000
#define LCD_WAIT_TSU1			1		/**< tsu1 >=   60ns					*/
#define LCD_WAIT_TSU			1		/**< tsu  >=   60ns					*/
#define LCD_WAIT_TW				1		/**< tw   >=  450ns					*/
#define LCD_WAIT_TH2			1		/**< th2  >=   10ns					*/
#define LCD_WAIT_TC				1		/**< tc   >= 1000ns					*/
/** @} */

/**
 * @name	Instruction Code
 * @{
 */
#define LCD_CLEAR_DISPLAY		0x01	/**< Clear Display					*/
#define LCD_RETURN_HOME			0x02	/**< Return Home					*/
#define LCD_ENTRY_MODE_SET		0x06	/**< Entry Mode Set					*/
#define LCD_DISPLAY_ON_OFF		0x0C	/**< Display ON/OFF Control			*/
#define LCD_CURSOR_OR_SHIFT		0x10	/**< Cursor or Display Shift		*/
#define LCD_FUNCTION_SET		0x38	/**< Function Set					*/
#define LCD_SET_CGRAM_ADDR		0x40	/**< Set CGRAM Address				*/
#define LCD_SET_DDRAM_ADDR		0x80	/**< Set DDRAM Address				*/
/** @} */

/*------------------------------------------*/
/*				ϐ					*/
/*------------------------------------------*/
/** es̐擪AhXe[u */
static const int addr_table[LCD_ROW_MAX] = {0x00, 0x40};

/*------------------------------------------*/
/*				O֐					*/
/*------------------------------------------*/
extern void wait_us(uint32_t us);

/*------------------------------------------*/
/*				֐					*/
/*------------------------------------------*/
static ibool	Lcd_BusyWait(void);
static uint8_t	Lcd_Read(void);
static void		Lcd_Write(uint8_t data);

/************************************************************************/
/**
 * @brief	LCD
 *
 * @retval	TRUE	I
 * @retval	FALSE	^CAEg
 */
ibool Lcd_Init(void)
{
	ibool	ret = FALSE;

	/* |[g */
	LCD_INIT();

	/* dOÑEFCg */
	wait_us(LCD_POWER_ON_WAIT);

	/* LCD̏ݒ */
	LCD_RS_LOW();
	Lcd_Write(LCD_FUNCTION_SET);
	if (Lcd_BusyWait())
	{
		Lcd_Write(LCD_DISPLAY_ON_OFF);
		if (Lcd_BusyWait())
		{
			Lcd_Write(LCD_CLEAR_DISPLAY);
			if (Lcd_BusyWait())
			{
				Lcd_Write(LCD_ENTRY_MODE_SET);
				if (Lcd_BusyWait())
				{
					Lcd_Write(LCD_CLEAR_DISPLAY);
					if (Lcd_BusyWait())
					{
						ret = TRUE;
					}
				}
			}
		}
	}

	return ret;
}

/************************************************************************/
/**
 * @brief	LCDNA
 *
 * @retval	TRUE	I
 * @retval	FALSE	ُI
 */
ibool Lcd_Clear(void)
{
	LCD_RS_LOW();
	Lcd_Write(LCD_CLEAR_DISPLAY);
	return Lcd_BusyWait();
}

/************************************************************************/
/**
 * @brief	o
 *
 * @param	column	[in]	
 * @param	row		[in]	s
 * @param	str		[in]	o͕ւ̃|C^
 *
 * @retval	TRUE	I
 * @retval	FALSE	ُI
 */
ibool Lcd_Puts(int column, int row, const char *str)
{
	if ((column < 0) || (LCD_COLUMN_MAX <= column) || (row < 0) || (LCD_ROW_MAX <= row) || (str == NULL))
	{
		return FALSE;
	}

	LCD_RS_LOW();
	Lcd_Write(LCD_SET_DDRAM_ADDR | (addr_table[row] + column));
	if (!Lcd_BusyWait())
	{
		return FALSE;
	}

	while ((*str != '\0') && (column < LCD_COLUMN_MAX))
	{
		LCD_RS_HIGH();
		Lcd_Write((uint8_t)*str);
		if (!Lcd_BusyWait())
		{
			return FALSE;
		}
		str++;
		column++;
	}

	return TRUE;
}

/************************************************************************/
/**
 * @brief	rW[҂
 *
 * @retval	TRUE	I
 * @retval	FALSE	ُI
 */
static ibool Lcd_BusyWait(void)
{
	int	i;

	LCD_RS_LOW();
	for (i = 0; i < LCD_BUSY_TIMEOUT; i++)
	{
		if (!(Lcd_Read() & 0x80))
		{
			return TRUE;
		}
	}

	return FALSE;
}

/************************************************************************/
/**
 * @brief	WX^[h
 *
 * @return	[hl
 */
static uint8_t Lcd_Read(void)
{
	uint8_t	ret;

	LCD_RW_HIGH();
	wait_us(LCD_WAIT_TSU);

	LCD_E_HIGH();
	wait_us(LCD_WAIT_TW);

	ret = LCD_READ();
	LCD_E_LOW();
	wait_us(LCD_WAIT_TC);

	return ret;
}

/************************************************************************/
/**
 * @brief	WX^Cg
 *
 * @param	data	[in]	Cgl
 */
static void Lcd_Write(uint8_t data)
{
	LCD_RW_LOW();
	wait_us(LCD_WAIT_TSU1);

	LCD_E_HIGH();
	LCD_WRITE(data);
	LCD_DIR_OUT();
	wait_us(LCD_WAIT_TW);

	LCD_E_LOW();
	wait_us(LCD_WAIT_TH2);

	LCD_DIR_IN();
	wait_us(LCD_WAIT_TC);
}
